#include "VSSniff.h"
#include "VSG72X.h" // VSG72XDLL

// ----------------------------------------------------------------------------
// VSSniff
// ----------------------------------------------------------------------------
VSSniff::VSSniff(void* owner)
{
  this->owner = owner;
  callList    = new VSCallList;
  // Packet count must be over 100
  // If packet count is under 100, call will be deleted.
  minCallPacketCount = 100;
  closeOldCallInterval = 5000; // 5 seconds

  VSG72XDLL::classInitialize();
}

VSSniff::~VSSniff()
{
  close();
  SAFE_DELETE(callList);

  VSG72XDLL::classFinalize();
}

bool VSSniff::open()
{
  callList->clear();
  return true;
}

bool VSSniff::close()
{
  callList->clear();
  return true;
}

void VSSniff::closeOldCall(DWORD tick)
{
  int i = 0;
  while (i < callList->count())
  {
    VSCall* call = callList->at(i);
    DWORD lastAccessTick = call->lastAccessTick;
    if (lastAccessTick!= 0 && lastAccessTick + closeOldCallInterval < tick)
    {
      LOG_INFO("[VSSniff.cpp] VSSniff::closeOldCall %s %s", call->from.c_str(), call->to.c_str());
      callList->remove(call);
    } else
    {
      i++;
    }
  }
}

void VSSniff::process(SnoopCapturer* capturer, SnoopIPPacket* packet)
{
  //
  // Close old call
  //
  if (closeOldCallInterval != 0)
  {
    DWORD tick =
      packet->pktHdr->ts.tv_sec * 1000 +
      packet->pktHdr->ts.tv_usec / 1000;
    closeOldCall(tick);
  }

  IP_HDR*  ipHdr  = packet->ipHdr;
  //
  // Retrieve srcIP, dstIP, srcPort, dstPort
  //
  TCP_HDR* tcpHdr;
  UDP_HDR* udpHdr;
  char*    data;
  int      dataLen;

  VSAddr srcAddr;
  VSAddr dstAddr;
  bool   isTCP;

  if (SnoopIP::isTCP(ipHdr, &tcpHdr))
  {
    if (!SnoopTCP::isData(ipHdr, tcpHdr, &data, &dataLen))
      return;
    srcAddr.ip   = ntohl(ipHdr->ip_src);
    srcAddr.port = ntohs(tcpHdr->th_sport);
    dstAddr.ip   = ntohl(ipHdr->ip_dst);
    dstAddr.port = ntohs(tcpHdr->th_dport);
    isTCP = true;
  } else
  if (SnoopIP::isUDP(ipHdr, &udpHdr))
  {
    if (!SnoopUDP::isData(ipHdr, udpHdr, &data, &dataLen))
      return;
    srcAddr.ip   = ntohl(ipHdr->ip_src);
    srcAddr.port = ntohs(udpHdr->uh_sport);
    dstAddr.ip   = ntohl(ipHdr->ip_dst);
    dstAddr.port = ntohs(udpHdr->uh_dport);
    isTCP = false;
  } else
    return;

  bool dataChanged = false;
  if (VSSessionMgr::process(this, packet, &srcAddr, &dstAddr, data, dataLen, &dataChanged))
  {
    if (dataChanged)
    {
      if (isTCP)
      {
        tcpHdr->th_sum = htons(SnoopTCP::checksum(ipHdr, tcpHdr));
      } else // UDP
      {
        udpHdr->uh_sum = htons(SnoopUDP::checksum(ipHdr, udpHdr));
      }
      // LOG_DEBUG("[VSSniff.cpp] VSSniff::process data changed"); // gilgil temp 2009.02.19
    }
    return;
  }
  VSCallMgr::process(this, packet, &srcAddr, &dstAddr, data, dataLen);
}

void VSSniff::load(VDXML* xml)
{
  minCallPacketCount = xml->getInt("minCallPacketCount", minCallPacketCount);
  closeOldCallInterval = xml->getInt("closeOldCallInterval", closeOldCallInterval);
}

void VSSniff::save(VDXML* xml)
{
  xml->setInt("minCallPacketCount", minCallPacketCount);
  xml->setInt("closeOldCallInterval", closeOldCallInterval);
}
