// ----------------------------------------------------------------------------
//
// VoIPSniffer version 2.0
//
// http://www.gilgil.net
//
// Copyright (c) NetworkLab Corporation All rights reserved
//
// ----------------------------------------------------------------------------

#ifndef __VSSNIFFFILE_H__
#define __VSSNIFFFILE_H__

#include "VSCall.h"
#include "VSSession.h"
#include "VSSniff.h"

#include "VDWAVRecord.h"
#include "SnoopDump.h"

// ----------------------------------------------------------------------------
// VSCallFile
// ----------------------------------------------------------------------------
class VSSniffFile;
class VSCallFile : public VSCall
{
public:
  VSSniffFile* sniffFile;
public:
  VSCallFile(VSSniffFile* sniffFile);
  virtual ~VSCallFile();
  virtual bool open();
  virtual bool close();
  virtual void changeCallInfo(string from, string to, VSAddr* fromAddr, VSAddr* toAddr);
  virtual void processPacket(SnoopIPPacket* packet);

public:
  bool callInfoChangedOnActive;
  
public:
  string       waveFileName;
  VDWAVRecord* wavRecord;
public:
  string getWaveFileName();
};

// ----------------------------------------------------------------------------
// VSSessionFile
// ----------------------------------------------------------------------------
class VSSessionFile : public VSSession
{
public:
  VSCallFile* callFile;
  char* mentBuffer;
  int   restMentBufferSize;
public:
  VSSessionFile(VSCallFile* callFile);
  virtual ~VSSessionFile();
  virtual bool open();
  virtual bool close();
public:
  virtual void processIPPacket(SnoopIPPacket* packet);
  virtual void processPCM(SnoopIPPacket* packet, char* pcm, int pcmLen, char* bit, int bitLen, bool* dataChanged);
};

// ----------------------------------------------------------------------------
// VSSniffFile
// ----------------------------------------------------------------------------
class VSSniffFile : public VSSniff
{
  friend class VSSessionFile;
public:
  VSSniffFile(void* owner = NULL);
  virtual ~VSSniffFile();
  virtual bool open();
  virtual bool close();

public:
  bool   saveCaptureFile;
  string captureFileFolder;
  bool   saveWaveFile;
  string waveFileFolder;
  bool   insertMent;
  string mentFileName;
  int    jitterTime;

protected:
  string     captureFileName;
  SnoopDump* dump;
  string     getCaptureFileName();
  char*      mentBuffer;
  int        mentBufferSize;

public:
  virtual void       process(SnoopCapturer* capturer, SnoopIPPacket* packet);
  virtual VSCall*    createNewCall();
  virtual VSSession* createNewSession(VSCall* call);

public:
  virtual void load(VDXML* xml);
  virtual void save(VDXML* xml);
};

#endif  __VSSNIFFFILE_H__
