#include "VSSniffGUI.h"

// ----------------------------------------------------------------------------
// VSCallGUI
// ----------------------------------------------------------------------------
VSCallGUI::VSCallGUI(VSSniffGUI* sniffGUI) : VSCallFile(sniffGUI)
{
  this->sniffGUI = sniffGUI;
  callForm = new TVSCallForm(sniffGUI->ownerForm);
  callForm->Parent = sniffGUI->showPanel;

  int i = 1;
  AnsiString componentName;
  while (true)
  {
    componentName = callForm->ClassName();
    componentName += IntToStr(i);
    if (sniffGUI->ownerForm->FindComponent(componentName) == NULL) break;
    i++;
  }
  callForm->Name = componentName;
  // callForm->Show(); // callForm is shown if packetCount is over minCallPacketCount
};

VSCallGUI::~VSCallGUI()
{
  close();
  if (!sniffGUI->ownerForm->Visible || packetCount < sniffFile->minCallPacketCount)
  {
    callForm->Close(); // Form is deleted automatically
  }
}

bool VSCallGUI::open()
{
  return VSCallFile::open();
}

bool VSCallGUI::close()
{
  bool res = VSCallFile::close();
  if (VDFile::fileExists(waveFileName))
  {
    callForm->sbtPlay->Hint = waveFileName.c_str();
    callForm->sbtPlay->Enabled = true;
  }
  callForm->sbtClose->Enabled = true;
  callForm->tmrMinimize->Enabled = true;
  return res;
}

void VSCallGUI::changeCallInfo(string from, string to, VSAddr* fromAddr, VSAddr* toAddr)
{
  VSCallFile::changeCallInfo(from, to, fromAddr, toAddr);
  callForm->lblCaption->Caption = (from + " > " + to).c_str();
}

void VSCallGUI::processPacket(SnoopIPPacket* packet)
{
  VSCallFile::processPacket(packet);
  if (packetCount == sniffFile->minCallPacketCount)
  {
    callForm->Show();
    callForm->Repaint();
  }
}

// ----------------------------------------------------------------------------
// VSSessionGUI
// ----------------------------------------------------------------------------
VSSessionGUI::VSSessionGUI(VSCallGUI* callGUI) : VSSessionFile(callGUI)
{
  this->callGUI = callGUI;
  TVSCallForm* callForm = callGUI->callForm;
  sessionFrame = new TVSSessionFrame(callForm);
  sessionFrame->Parent = callForm->pnlMain;

  int i = 1;
  AnsiString componentName;
  while (true)
  {
    componentName = callForm->ClassName();
    componentName += IntToStr(i);
    if (callForm->FindComponent(componentName) == NULL) break;
    i++;
  }
  sessionFrame->Name = componentName;
  callForm->Height = callForm->Height + sessionFrame->Height;
}

VSSessionGUI::~VSSessionGUI()
{
  close();
  // sessionFrame is deleted automatically when callForm is deleted
}

bool VSSessionGUI::open()
{
  return VSSessionFile::open();
}

bool VSSessionGUI::close()
{
  bool res = VSSessionFile::close();
  if (callGUI->sniffGUI->showGraph)
    showAudioData(sessionFrame->Handle, 16, 0, 0, RGB(0, 255, 255), RGB(0x40, 0x40, 0x40));
  return res;
}

void VSSessionGUI::processPCM(SnoopIPPacket* packet, char* pcm, int pcmLen, char* bit, int bitLen, bool* dataChanged)
{
  VSSessionFile::processPCM(packet, pcm, pcmLen, bit, bitLen, dataChanged);
  if (callGUI->sniffGUI->showGraph)
    showAudioData(sessionFrame->Handle, 16, pcm, pcmLen, RGB(0, 255, 255), RGB(0x40, 0x40, 0x40));
}

// ----------------------------------------------------------------------------
// VSSniffGUI
// ----------------------------------------------------------------------------
VSSniffGUI::VSSniffGUI(void* owner) : VSSniffFile(owner)
{
  ownerForm = NULL;
  showPanel = NULL;
  showGraph = true;
}

VSSniffGUI::~VSSniffGUI()
{
}

VSCall* VSSniffGUI::createNewCall()
{
  VSCallGUI* res = new VSCallGUI(this);
  return res;
}

VSSession* VSSniffGUI::createNewSession(VSCall* call)
{
  return new VSSessionGUI((VSCallGUI*)call);
}

void VSSniffGUI::load(VDXML* xml)
{
  VSSniffFile::load(xml);
  showGraph = xml->getBool("showGraph", showGraph);
}

void VSSniffGUI::save(VDXML* xml)
{
  VSSniffFile::save(xml);
  xml->setBool("showGraph", showGraph);
}
