//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormMain.h"
#include "FormOption.h"
#include "FormAbout.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "FrameSnoopSessionList"
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
void __fastcall TMainForm::WndProc(TMessage &Message)
{
  switch (Message.Msg)
  {
    case WM_MY_CLOSE:
      btnClose->Click();
      break;
    default:
      TForm::WndProc(Message);
  }
}

void TMainForm::initializeControl()
{
  SnoopSessionListFrame1->Align = alClient;
  pnlMain->BevelOuter = bvNone;
  pnlMain->Align = alClient;
  pnlMain->Color= clBlack;

  snoopARPSpoof.owner = this;
  snoopARPSpoof.onClose = _snoopCapturer_OnClose;
  snoopARPSpoof.onCapture = _snoopARPSpoof_OnCapture;

  snoopAdapter.owner = this;
  snoopAdapter.onClose = _snoopCapturer_OnClose;
  snoopAdapter.onCapture = _snoopCapturer_OnCapture;

  snoopFile.owner = this;
  snoopFile.onClose = _snoopCapturer_OnClose;
  snoopFile.onCapture = _snoopCapturer_OnCapture;

  sniff = new VSSniffGUI(this);
  sniff->ownerForm = this;
  sniff->showPanel = pnlMain;
}

void TMainForm::finalizeControl()
{
  SAFE_DELETE(sniff);
}

void TMainForm::loadControl()
{
  // WindowPos
  if (!global->loadWindowPos(Handle, "MainForm"))
  {
    Left = 0;
    Top = 0;
  }
  if (!global->loadWindowPos(pnlHostList->Handle, "MainFormLeft"))
  {
    pnlHostList->Width = 333;
  }
  pnlHostList->Left = 0;
  pnlHostList->Top = 0;

  // SessionListFrame
  global->loadSessionListFrame(SnoopSessionListFrame1);

  // Other
  global->loadOther();
  chkRelay->Checked = global->relay;
}

void TMainForm::saveControl()
{
  // WindowPos
  global->saveWindowPos(Handle, "MainForm");
  global->saveWindowPos(pnlHostList->Handle, "MainFormLeft");

  // SessionListFrame
  global->saveSessionListFrame(SnoopSessionListFrame1);

  // Other
  global->relay = chkRelay->Checked;
  global->saveOther();
}

void TMainForm::setControl()
{
  bool scanning = SnoopSessionListFrame1->SnoopHostListFrame1->active();
  bool active = snoopARPSpoof.active() || snoopAdapter.active() || snoopFile.active();
  btnOpen->Enabled = !scanning && !active;
  btnClose->Enabled = !scanning && active;
  miFile->Enabled = btnOpen->Enabled;
}

void TMainForm::_snoopCapturer_OnClose(void* sender)
{
  SnoopCapturer* snoopCapturer = (SnoopCapturer*)sender;
  TMainForm* MainForm = (TMainForm*)snoopCapturer->owner;
  PostMessage(MainForm->Handle, WM_MY_CLOSE, 0, 0);
}

void TMainForm::_snoopARPSpoof_OnCapture(
  SnoopARPSpoof*       snoopARPSpoof,
  SnoopARPSpoofPacket* packet)
{
  TMainForm* MainForm = (TMainForm*)snoopARPSpoof->owner;
  MainForm->sniff->process(snoopARPSpoof, packet);
  packet->relay = MainForm->chkRelay->Checked;
}

void TMainForm::_snoopCapturer_OnCapture(
  SnoopCapturer* capturer,
  PKT_HDR*       pktHdr,
  char*          pktData)
{
  SnoopIPPacket packet;
  packet.pktHdr  = pktHdr;
  packet.pktData = pktData;
  if (!SnoopIP::parse(&packet)) return;

  TMainForm* MainForm = (TMainForm*)capturer->owner;
  MainForm->sniff->process(capturer, &packet);
}
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormCreate(TObject *Sender)
{
  initializeControl();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormShow(TObject *Sender)
{
  loadControl();
  setControl();
  int count = SnoopSessionListFrame1->SnoopHostListFrame1->lvHostList->Items->Count;
  if (count == 0)
    SnoopSessionListFrame1->SnoopHostListFrame1->btnScan->Click();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
  saveControl();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
  finalizeControl();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::btnOpenClick(TObject *Sender)
{
  global->loadSnoopARPSpoof(&snoopARPSpoof);
  global->loadSnoopAdapter(&snoopAdapter);
  SnoopSessionListFrame1->loadControl(&snoopARPSpoof.sessionList, ip_to_str(global->netInfo.gateway));

  if (snoopARPSpoof.sessionList.count() == 0) // Open snoopAdapter
  {
    if (!snoopAdapter.open())
    {
      Application->MessageBoxA(snoopAdapter.error.msg, "Error", MB_ICONERROR);
      goto _error;
    }
  } else // Open snoopARPSpoof
  {
    if (!snoopARPSpoof.open())
    {
      Application->MessageBoxA(snoopARPSpoof.error.msg, "Error", MB_ICONERROR);
      goto _error;
    }
  }
  global->loadVoIPSniff(sniff);
  if (!sniff->open())
  {
    Application->MessageBoxA("Can not open sniff", "Error", MB_ICONERROR);
    goto _error;
  }
  setControl();
  return;
_error:
  snoopAdapter.close();
  snoopARPSpoof.close();
  sniff->close();
  setControl();  
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::btnCloseClick(TObject *Sender)
{
  snoopARPSpoof.close();
  snoopAdapter.close();
  snoopFile.close();
  sniff->close();
  setControl();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::btnOptionClick(TObject *Sender)
{
  if (OptionForm == NULL)
    OptionForm = new TOptionForm(Application);
  OptionForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SnoopHostListFrame1btnScanClick(TObject *Sender)
{
  SnoopSessionListFrame1->SnoopHostListFrame1->scan(
    global->adapterIndex,
    global->netInfo.startIP,
    global->netInfo.endIP
  );
  setControl();  
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SnoopHostListFrame1tmrCloseTimer(TObject *Sender)
{
  SnoopSessionListFrame1->SnoopHostListFrame1->tmrCloseTimer(Sender);
  setControl();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::miFileClick(TObject *Sender)
{
  if (!OpenDialog1->Execute()) return;

  global->loadSnoopFile(&snoopFile);
  global->saveSnoopFile(&snoopFile);
//  snoopFile.autoCapture = false;

  snoopFile.fileName = OpenDialog1->FileName.c_str();
  if (!snoopFile.open())
  {
    Application->MessageBoxA(snoopFile.error.msg, "Error", MB_ICONERROR);
  }
  global->loadVoIPSniff(sniff);
  sniff->saveCaptureFile = false;
  sniff->insertMent = false;
  sniff->showGraph = snoopFile.timeSync;
  if (!sniff->open())
  {
    Application->MessageBoxA("Can not open sniff", "Error", MB_ICONERROR);
    btnClose->Click();
  }
  setControl();


  // ----- gilgil temp 2009.09.21 -----
  /*
  while (true)
  {
    SnoopIPPacket packet;
    int res = global->snoopFile.read(&packet);
    if (res == 0) continue;
    if (res < 0) break;
    sniff->process(&snoopIP, &packet);
  }
  snoopIP.close();
  sniff->close();
  */
  // ----------------------------------
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::miOptionClick(TObject *Sender)
{
  btnOption->Click();  
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::miHelpClick(TObject *Sender)
{
  if (AboutForm == NULL) AboutForm = new TAboutForm(Application);
  AboutForm->ShowModal();  
}
//---------------------------------------------------------------------------

