#include "Global.h"
#include "VDXMLAux.h"
#include "VDLogFactory.h"
#include "VDLogList.h"

Global* global = NULL;

Global::Global()
{
  arpSpoof.autoCapture = true;
  arpSpoof.threadSafe = true;
  adapter.autoCapture = true;
  adapter.threadSafe = true;
  relay = true;

  load();
  initializeLog();
}

Global::~Global()
{
  save();
  finalizeLog();
}

void Global::load()
{
  xmlFile.loadFromFile();
}

void Global::save()
{
  xmlFile.saveToFile();
}

bool Global::isOk()
{
  bool res = true;

  loadSnoopARPSpoof(&arpSpoof);
  if (adapterIndex == -1) res = false;

  loadNetInfo(&netInfo);
  if (netInfo.ip == 0) res = false;
  if (netInfo.mac.isClear()) res = false;
  if (netInfo.subnetMask == 0) res = false;
  if (netInfo.gateway == 0) res = false;
  if (netInfo.startIP == 0) res = false;
  if (netInfo.endIP == 0) res = false;

  return res;
}

void Global::initializeLog()
{
  VDXML* xml = xmlFile.root->gotoChild("Log");
  VDLog* log = VDLogFactory::createByXML(xml);

  if (log == NULL)
  {
    VDLogList* logList = new VDLogList;
    logList->items.add(VDLogFactory::createByURI("udp"));
    logList->items.add(VDLogFactory::createByURI("file://log/"));
    log = logList;
    VDLogFactory::save(log, xml);
    save();
  }
  VDLog::changeLog(log);
}

void Global::finalizeLog()
{
  // VDLog::changeLog(NULL);
}

bool Global::loadWindowPos(HWND hwnd, char* name)
{
  VDXML* xml = xmlFile.root->gotoChild("WindowPos")->gotoChild(name);
  return VDXMLAux::loadWindowPos(hwnd, xml); 
}

bool Global::saveWindowPos(HWND hwnd, char* name)
{
  VDXML* xml = xmlFile.root->gotoChild("WindowPos")->gotoChild(name);
  return VDXMLAux::saveWindowPos(hwnd, xml); 
}

int Global::getAdapterIndex()
{
  return arpSpoof.adapterIndex;
}

void Global::setAdapterIndex(int value)
{
  arpSpoof.adapterIndex = value;
  adapter.adapterIndex = value;
  netInfo.adapterIndex = value;
}

void Global::loadSnoopARPSpoof(SnoopARPSpoof* arpSpoof)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopARPSpoof");
  arpSpoof->load(xml);
}

void Global::saveSnoopARPSpoof(SnoopARPSpoof* arpSpoof)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopARPSpoof");
  arpSpoof->save(xml);
}

void Global::loadSnoopAdapter(SnoopAdapter* adapter)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopAdapter");
  adapter->load(xml);
}

void Global::saveSnoopAdapter(SnoopAdapter* adapter)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopAdapter");
  adapter->save(xml);
}

void Global::loadSnoopFile(SnoopFile* snoopFile)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopFile");
  snoopFile->load(xml);
}

void Global::saveSnoopFile(SnoopFile* snoopFile)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopFile");
  snoopFile->save(xml);
}

void Global::loadNetInfo(SnoopNetInfo* netInfo)
{
  VDXML* xml = xmlFile.root->gotoChild("NetInfo");
  netInfo->load(xml);
}

void Global::saveNetInfo(SnoopNetInfo* netInfo)
{
  VDXML* xml = xmlFile.root->gotoChild("NetInfo");
  netInfo->save(xml);
}

void Global::loadSessionListFrame(TSnoopSessionListFrame* frame)
{
  VDXML* xml = xmlFile.root->gotoChild("SessionListFrame");
  frame->load(xml);
}

void Global::saveSessionListFrame(TSnoopSessionListFrame* frame)
{
  VDXML* xml = xmlFile.root->gotoChild("SessionListFrame");
  frame->save(xml);
}

void Global::loadVoIPSniff(VSSniff* voipSniff)
{
  VDXML* xml = xmlFile.root->gotoChild("VSSniff");
  voipSniff->load(xml);
}

void Global::saveVoIPSniff(VSSniff* voipSniff)
{
  VDXML* xml = xmlFile.root->gotoChild("VSSniff");
  voipSniff->save(xml);
}

void Global::loadOther()
{
  VDXML* xml = xmlFile.root->gotoChild("Other");
  relay = xml->getBool("relay", relay);
}

void Global::saveOther()
{
  VDXML* xml = xmlFile.root->gotoChild("Other");
  xml->setBool("relay", relay);
}

bool Global::isSameLanIP(IP ip)
{
  return netInfo.isSameLanIP(ip);
}
