//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormMain.h"
//---------------------------------------------------------------------------
#include "FormBasicOption.h"
#include "FormRegister.h"
#include "FormAbout.h"
#include "FormQuiz.h"
#include "FormHost.h"
#include "FormSniff.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "ImageButton"
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
void __fastcall TMainForm::WndProc(TMessage &Message)
{
  switch (Message.Msg)
  {
    case WM_MY_CLOSE:
     this->ibtStop->Click();
      break;
    default:
      TForm::WndProc(Message);
  }
}

void TMainForm::initializeControl()
{
  // BorderStyle = bsNone;
  
  pnlTop->BevelOuter = bvNone;
  pnlTop->Align = alTop;
  pnlTop->Height = imgTop->Height;
  imgTop->Align = alTop;
  imgTop->Stretch = true;

  pnlBottom->BevelOuter = bvNone;
  pnlBottom->Align = alBottom;
  pnlBottom->Height = imgBottom->Height;
  imgBottom->Align = alTop;
  imgBottom->Stretch = true;

  pnlMain->BevelOuter = bvNone;
  pnlMain->Align = alClient;

  sbtOption->Flat = true;

  ibtStart->Left = ibtScan->Left;
  ibtStop->Left = ibtClear->Left;
  ibtRegister->Left = ibtScan->Left;

  snoopARPSpoof.owner = this;
  snoopARPSpoof.onClose = _snoopCapturer_OnClose;
  snoopARPSpoof.onCapture = _snoopARPSpoof_OnCapture;

  snoopAdapter.owner = this;
  snoopAdapter.onClose = _snoopCapturer_OnClose;
  snoopAdapter.onCapture = _snoopAdapter_OnCapture;

  sniff = new VSSniffGUI(this);
}

void TMainForm::finalizeControl()
{
  SAFE_DELETE(sniff);
}

void TMainForm::loadControl()
{
  // WindowPos
  if (!global->loadWindowPos(Handle, "MainForm"))
  {
    Left = (Screen->Width - Width) / 2;
    Top = (Screen->Height - Height) / 2;
  }

  quizResolved = false;

  //
  // Before go to step vssAbout, launch web site for the first time.
  //
  if (!g_reg->isRegistered())
  {
    if (QuizForm == NULL)
    {
      QuizForm = new TQuizForm(Application);
      QuizForm->Parent = pnlMain;
      QuizForm->Align = alClient;
      QuizForm->Show();
    }
  }

  step = vssNone;
  gotoStep(vssAbout);
}

void TMainForm::saveControl()
{
  // WindowPos
  global->saveWindowPos(Handle, "MainForm");
}

void TMainForm::setControl()
{
  //
  // Scan and Clear button is visible only when step is vssHost
  //
  ibtScan->Visible = step == vssHost;
  ibtClear->Visible = step == vssHost;

  //
  // Registerbutton is visible only when step is vssHost
  //
  ibtRegister->Visible = step == vssQuiz;

  //
  // Start and Stop button is visible only when step is vssSniff
  //
  ibtStart->Visible = step == vssSniff;
  ibtStop->Visible = step == vssSniff;

  //
  // Start and Stop button
  //
  bool active = snoopARPSpoof.active() || snoopAdapter.active();
  ibtStart->Enabled = !active;
  ibtStop->Enabled = active;

  //
  // Prev and Next button
  //
  ibtPrev->Enabled = step != vssAbout;

  if (step == vssSniff || (step == vssQuiz && !quizResolved))
  {
    ibtNext->Enabled = false;
  } else
  {
    ibtNext->Enabled = true;
  }


}

void TMainForm::gotoStep(VSStep step)
{
  if (step == this->step) return;

  switch (step)
  {
    case vssNone:
      break;
    case vssAbout:
      if (AboutForm == NULL)
      {
        AboutForm = new TAboutForm(Application);
        AboutForm->Parent = pnlMain;
        AboutForm->Align = alClient;
        AboutForm->Show();
      }
      lblTitle->Caption = "About VoIP Sniffer Basic";
      AboutForm->BringToFront();
      break;
    case vssQuiz:
      if (QuizForm == NULL)
      {
        QuizForm = new TQuizForm(Application);
        QuizForm->Parent = pnlMain;
        QuizForm->Align = alClient;
        QuizForm->Show();
      }
      lblTitle->Caption = "Mark the correct answer to use VoIP Sniffer Basic";
      QuizForm->BringToFront();
      break;
    case vssHost:
      if (HostForm == NULL)
      {
        HostForm = new THostForm(Application);
        HostForm->Parent = pnlMain;
        HostForm->Align = alClient;
        HostForm->Show();
      }
      lblTitle->Caption = "Select host to sniff";
      HostForm->BringToFront();
      break;
    case vssSniff:
      if (SniffForm == NULL)
      {
        SniffForm = new TSniffForm(Application);
        SniffForm->Parent = pnlMain;
        SniffForm->Align = alClient;
        SniffForm->Show();
        sniff->ownerForm = SniffForm;
        SniffForm->Align = alClient;
        sniff->showPanel = SniffForm->pnlMain;
      }
      lblTitle->Caption = "Click Start to sniff";
      SniffForm->BringToFront();
      break;
  }

  this->step = step;
  setControl();
}

void TMainForm::_snoopCapturer_OnClose(void* sender)
{
  SnoopCapturer* snoopCapturer = (SnoopCapturer*)sender;
  TMainForm* MainForm = (TMainForm*)snoopCapturer->owner;
  PostMessage(MainForm->Handle, WM_MY_CLOSE, 0, 0);
}

void TMainForm::_snoopARPSpoof_OnCapture(
    SnoopARPSpoof*       snoopARPSpoof,
    SnoopARPSpoofPacket* packet)
{
  TMainForm* MainForm = (TMainForm*)snoopARPSpoof->owner;
  MainForm->sniff->process(snoopARPSpoof, packet);
  packet->relay = true;
}

void TMainForm::_snoopAdapter_OnCapture(
    SnoopCapturer* capturer,
    PKT_HDR*       pktHdr,
    char*          pktData)
{
  SnoopIPPacket packet;
  packet.pktHdr  = pktHdr;
  packet.pktData = pktData;
  if (!SnoopIP::parse(&packet)) return;

  TMainForm* MainForm = (TMainForm*)capturer->owner;
  MainForm->sniff->process(capturer, &packet);
}
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormCreate(TObject *Sender)
{
  initializeControl();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormShow(TObject *Sender)
{
  if (Tag == 0) // if it is first show
  {
    loadControl();
    setControl();
    Tag = 1;
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
  ibtStop->Click();
  saveControl();
  if (AboutForm != NULL) AboutForm->Close();
  if (QuizForm != NULL) QuizForm->Close();
  if (HostForm != NULL) HostForm->Close();  
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
  finalizeControl();
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::sbtOptionClick(TObject *Sender)
{
  if (BasicOptionForm == NULL)
  {
    BasicOptionForm = new TBasicOptionForm(Application);
  }
  BasicOptionForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ibtPrevClick(TObject *Sender)
{
  switch (step)
  {
    case vssNone:  break;
    case vssAbout: break;
    case vssQuiz:  gotoStep(vssAbout); break;
    case vssHost:
      if (g_reg->isRegistered())
        gotoStep(vssAbout);
      else
        gotoStep(vssQuiz);
      break;
    case vssSniff: gotoStep(vssHost);  break;
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ibtNextClick(TObject *Sender)
{
  switch (step)
  {
    case vssNone:  break;
    case vssAbout:
      if (g_reg->isRegistered())
        gotoStep(vssHost);
      else
        gotoStep(vssQuiz);
      break;
    case vssQuiz:  gotoStep(vssHost);  break;
    case vssHost:  gotoStep(vssSniff); break;
    case vssSniff: break;
  }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ibtScanClick(TObject *Sender)
{
  if (HostForm == NULL) return;
  HostForm->SnoopSessionListFrame1->SnoopHostListFrame1->scan(
    global->adapterIndex,
    global->netInfo.startIP,
    global->netInfo.endIP
  );
  setControl();
  HostForm->setControl();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ibtClearClick(TObject *Sender)
{
  if (HostForm == NULL) return;
  HostForm->SnoopSessionListFrame1->SnoopHostListFrame1->btnClear->Click();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ibtStartClick(TObject *Sender)
{
  global->loadSnoopARPSpoof(&snoopARPSpoof);
  global->loadSnoopAdapter(&snoopAdapter);
  HostForm->SnoopSessionListFrame1->loadControl(&snoopARPSpoof.sessionList, ip_to_str(global->netInfo.gateway));

  if (snoopARPSpoof.sessionList.count() == 0) // Open snoopAdapter
  {
    if (!snoopAdapter.open())
    {
      Application->MessageBoxA(snoopAdapter.error.msg, "Error", MB_ICONERROR);
      goto _error;
    }
  } else // Open snoopARPSpoof
  {
    if (!snoopARPSpoof.open())
    {
      Application->MessageBoxA(snoopARPSpoof.error.msg, "Error", MB_ICONERROR);
      goto _error;
    }
  }
  global->loadVoIPSniff(sniff);
  if (!sniff->open())
  {
    Application->MessageBoxA("Can not open sniff", "Error", MB_ICONERROR);
    goto _error;
  }
  setControl();
  return;
_error:
  snoopAdapter.close();
  snoopARPSpoof.close();
  sniff->close();
  setControl();  
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ibtStopClick(TObject *Sender)
{
  snoopARPSpoof.close();
  snoopAdapter.close();
  sniff->close();
  setControl();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ibtRegisterClick(TObject *Sender)
{
  if (RegisterForm == NULL)
  {
    RegisterForm = new TRegisterForm(Application);
  }
  RegisterForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::MyShow(TMessage &Msg)
{
  LOG_DEBUG("TMainForm::MyShow %d", this->WindowState);
  Hide();
  Show();
  BringToFront();
}
