#include "GlobalBasic.h"

GlobalBasic::GlobalBasic()
{
  gkURL = "http://vs.gilgil.net/gk/";
}

GlobalBasic::~GlobalBasic()
{
}

void GlobalBasic::loadGateKeeper(GateKeeper* gk)
{
  VDXML* xml = xmlFile.root->gotoChild("GateKeeper");
  gkURL = xml->getStr("URL", gkURL);
}

void GlobalBasic::saveGateKeeper(GateKeeper* gk)
{
  VDXML* xml = xmlFile.root->gotoChild("GateKeeper");
  xml->setStr("URL", gkURL);
}

string encodeStrToHex(string str)
{
  int strLen = str.length();
  int hexLen = strLen * 2;
  VDRC4 rc4;
  int i;

  //
  // Initializer rc4
  //
  rc4.key = "gilgil";
  rc4.keyLen = 6;
  rc4.open();

  //
  // Translate str > cipher
  //
  string cipher = str;
  i = rc4.trans((char*)cipher.c_str(), cipher.length());
  if (i == VDERR_FAIL)
  {
    LOG_ERROR("[GlobalBasic.cpp] encodeString(%s) return VDERR_FAIL", str.c_str());
    return "";
  }
  rc4.close();

  //
  // cipher > hex
  //
  string hex;
  hex.resize(hexLen);
  i = bin_to_hex((char*)cipher.c_str(), strLen, (char*)hex.c_str(), hexLen);
  if (i == -1)
  {
    LOG_ERROR("[GlobalBasic.cpp] encodeStrToHex bin_to_hex return -1");
    return "";
  }

  return hex;
}

string decodeHexToStr(string hex)
{
  int hexLen = hex.length();
  int strLen = hexLen / 2;
  VDRC4 rc4;
  int i;

  //
  // hex > cipher
  //
  string cipher;
  cipher.resize(strLen);
  i = hex_to_bin((char*)hex.c_str(), hexLen, (char*)cipher.c_str(), strLen);
  if (i == -1)
  {
    LOG_ERROR("[GlobalBasic.cpp] decodeHexToStr hex_to_bin return -1");
    return "";
  }

  //
  // Initializer rc4
  //
  rc4.key = "gilgil";
  rc4.keyLen = 6;
  rc4.open();

  //
  // Translate cipher > str
  //
  string str = cipher;
  i = rc4.trans((char*)str.c_str(), str.length());
  if (i == VDERR_FAIL)
  {
    LOG_ERROR("[GlobalBasic.cpp] encodeString(%s) return VDERR_FAIL", str.c_str());
    return "";
  }
  rc4.close();

  return str;
}

#define UNIT_SIZE MD5_DATA_SIZE
string getMD5HexChecksum(string fileName)
{
  VDFile file;
  VDMD5 md5;

  char data[UNIT_SIZE];
  char checksum[MD5_DIGEST_SIZE];
  char checksumBin[MD5_DIGEST_SIZE * 2 + 1];

  if (!VDFile::fileExists(fileName))
  {
    LOG_ERROR("[GlobalBasic.cpp] file not exists(%s)", fileName.c_str());
    return "";
  }

  file.mode = vdmRead;
  file.fileName = fileName;
  if (!file.open())
  {
    LOG_ERROR("[GlobalBasic.cpp] can not open file %s", file.error.msg);
    return "";
  }

  md5.open();
  while (true)
  {
    int readLen = file.read(data, UNIT_SIZE);
    if (readLen == VDERR_FAIL) // may be end of file
      break;
    md5.write(data, readLen);
  }

  md5.read(checksum, MD5_DIGEST_SIZE);
  md5.close();

  bin_to_hex(checksum, MD5_DIGEST_SIZE, checksumBin, MD5_DIGEST_SIZE * 2);
  checksumBin[MD5_DIGEST_SIZE * 2] = '\0';

  string res = checksumBin;
  return res;
}
