#include "Global.h"
#include "VDXMLAux.h"
#include "VDLogFactory.h"

Global* global = NULL;

Global::Global()
{
  // VSSniff
  voipSniff.saveCaptureFile = false;
  voipSniff.insertMent = false;

  // DB
  databaseName = extractFilePath(application->exeName()) + "db\\VOIPSNIFFER.fdb";
  params = "user_name=SYSDBA\r\npassword=masterkey";

  load();
  initializeLog();
}

Global::~Global()
{
  save();
  finalizeLog();
}

void Global::load()
{
  xmlFile.loadFromFile();
}

void Global::save()
{
  xmlFile.saveToFile();
}

bool Global::isOk()
{
  bool res = true;

  loadSnoopAdapter(&adapter);
  if (adapterIndex == -1) res = false;

  loadNetInfo(&netInfo);
  if (netInfo.ip == 0) res = false;
  if (netInfo.mac.isClear()) res = false;
  if (netInfo.subnetMask == 0) res = false;
  if (netInfo.gateway == 0) res = false;
  if (netInfo.startIP == 0) res = false;
  if (netInfo.endIP == 0) res = false;

  return res;
}

void Global::initializeLog()             
{
  VDXML* xml = xmlFile.root->gotoChild("Log");
  VDLog* log = VDLogFactory::createByXML(xml);

  if (log == NULL)
  {
    VDLogList* logList = new VDLogList;
    logList->items.add(VDLogFactory::createByURI("udp"));
    logList->items.add(VDLogFactory::createByURI("file://log/"));
    log = logList;
    VDLogFactory::save(log, xml);
    save();
  }
  VDLog::changeLog(log);
}

void Global::finalizeLog()
{
#ifndef VD_USE_CLASS_INITIALIZER
  VDLog::changeLog(NULL);
#endif  VD_USE_CLASS_INITIALIZER
}

bool Global::loadWindowPos(HWND hwnd, char* name)
{
  VDXML* xml = xmlFile.root->gotoChild("WindowPos")->gotoChild(name);
  return VDXMLAux::loadWindowPos(hwnd, xml); 
}

bool Global::saveWindowPos(HWND hwnd, char* name)
{
  VDXML* xml = xmlFile.root->gotoChild("WindowPos")->gotoChild(name);
  return VDXMLAux::saveWindowPos(hwnd, xml); 
}

int Global::getAdapterIndex()
{
  return adapter.adapterIndex;
}

void Global::setAdapterIndex(int value)
{
  adapter.adapterIndex = value;
  netInfo.adapterIndex = value;
}

void Global::loadSnoopAdapter(SnoopAdapter* adapter)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopAdapter");
  adapter->load(xml);
}

void Global::saveSnoopAdapter(SnoopAdapter* adapter)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopAdapter");
  adapter->save(xml);
}

void Global::loadNetInfo(SnoopNetInfo* netInfo)
{
  VDXML* xml = xmlFile.root->gotoChild("NetInfo");
  netInfo->load(xml);
}

void Global::saveNetInfo(SnoopNetInfo* netInfo)
{
  VDXML* xml = xmlFile.root->gotoChild("NetInfo");
  netInfo->save(xml);
}

void Global::loadVoIPSniff(VSSniff* voipSniff)
{
  VDXML* xml = xmlFile.root->gotoChild("VSSniff");
  voipSniff->load(xml);
}

void Global::saveVoIPSniff(VSSniff* voipSniff)
{
  VDXML* xml = xmlFile.root->gotoChild("VSSniff");
  voipSniff->save(xml);
}

void Global::loadDB()
{
  VDXML* xml = xmlFile.root->gotoChild("DB");
  databaseName = xml->getStr("databaseName", databaseName);
  params = xml->getStr("params", params);
}

void Global::saveDB()
{
  VDXML* xml = xmlFile.root->gotoChild("DB");
  xml->setStr("databaseName", databaseName);
  xml->setStr("params", params);
}
