#include "FormMain.h"
#include "Main.h"

_Main::_Main()
{
  LOG_DEBUG("[Main.cpp] _Main::_Main");
  adapter = new SnoopAdapter(this);
  sniff   = new VSSniffDB(this);

  adapter->onClose   =  _snoopCapturer_OnClose;
  adapter->onCapture = _snoopCapturer_OnCapture;
}

_Main::~_Main()
{
  LOG_DEBUG("[Main.cpp] _Main::~_Main");
  close();
  SAFE_DELETE(adapter);
  SAFE_DELETE(sniff);
}

void _Main::_snoopCapturer_OnClose(void* sender)
{
  LOG_WARN("[Main.cpp] _Main::_snoopCapturer_OnClose beg post"); // gilgil temp 2009.10.08
  // SnoopAdapter* snoopAdapter = (SnoopAdapter*)sender;
  PostMessage(MainForm->Handle, WM_MY_CLOSE, 0, 0);
  LOG_WARN("[Main.cpp] _Main::_snoopCapturer_OnClose end"); // gilgil temp 2009.10.08
}

void _Main::_snoopCapturer_OnCapture(
  SnoopCapturer* capturer,
  PKT_HDR*       pktHdr,
  char*          pktData)
{
  SnoopIPPacket packet;
  packet.pktHdr  = pktHdr;
  packet.pktData = pktData;
  if (!SnoopIP::parse(&packet)) return;

  _Main* main = (_Main*)capturer->owner;
  main->sniff->process(capturer, &packet);
}

bool _Main::active()
{
  bool res = false;
  if (adapter != NULL) res = adapter->active();
  return res;
}

bool _Main::open()
{
  if (active())
  {
    LOG_ERROR("[Main.cpp] _Main::open already opened state");
    goto _error;
  }

  global->loadSnoopAdapter(adapter);
  LOG_DEBUG("[Main.cpp] _Main::open adapter->adapterIndex=%d(%s)",
    adapter->adapterIndex, snoopGetAdapterInfo(adapter->adapterIndex)->description);
  if (!adapter->open())
  {
    LOG_ERROR("[Main.cpp] _Main::open adapter->open return false(%s)", adapter->error.msg);
    goto _error;
  }

  global->loadVoIPSniff(sniff);
  if (!sniff->open())
  {
    LOG_ERROR("[Main.cpp] _Main::open sniff->open return false");
    goto _error;
  }

  return true;
_error:
  close();
  return false;
}

bool _Main::close()
{
  adapter->close();
  sniff->close();
  return true;
}
