#include "VSSniffDB.h"
#include "DataModule.h"

// ----------------------------------------------------------------------------
// VSCallDB
// ----------------------------------------------------------------------------
VSCallDB::VSCallDB(VSSniffDB* sniffDB) : VSCallFile(sniffDB)
{
  // LOG_TRACE("[VSSniffDB.cpp] VSCallDB::VSCallDB"); // gilgil temp 2009.10.05
}

VSCallDB::~VSCallDB()
{
  // LOG_TRACE("[VSSniffDB.cpp] VSCallDB::~VSCallDB"); // gilgil temp 2009.10.05
  close();
}

bool VSCallDB::open()
{
  // LOG_TRACE("[VSSniffDB.cpp] VSCallDB::open"); // gilgil temp 2009.10.05
  return VSCallFile::open();
}

bool VSCallDB::close()
{
  // LOG_TRACE("[VSSniffDB.cpp] VSCallDB::close filename=%s", waveFileName.c_str()); // gilgil temp 2009.10.05
  if (!VSCallFile::close()) return false;
  if (!VDFile::fileExists(waveFileName))
  {
    LOG_DEBUG("[VSSniffDB.cpp] VSCallDB::close file(%s) does not exist", waveFileName.c_str());
    return true;
  }
  // LOG_DEBUG("[VSSniffDB.cpp] VSCallDB::close file(%s) exists", waveFileName.c_str()); // gilgil temp 2009.10.05

  string beginTime;
  string endTime;
  char dateTime[256];
  sprintf(dateTime, "%04d%02d%02d%02d%02d%02d",
    begin_tm.tm_year + 1900,
    begin_tm.tm_mon + 1,
    begin_tm.tm_mday,
    begin_tm.tm_hour,
    begin_tm.tm_min,
    begin_tm.tm_sec);
  LOG_TRACE("[VSSniffDB.cpp] VSCallDB::close begin dateTime=%s", dateTime);
  beginTime = dateTime;

  sprintf(dateTime, "%04d%02d%02d%02d%02d%02d",
    end_tm.tm_year + 1900,
    end_tm.tm_mon + 1,
    end_tm.tm_mday,
    end_tm.tm_hour,
    end_tm.tm_min,
    end_tm.tm_sec);
  LOG_TRACE("[VSSniffDB.cpp] VSCallDB::close end  dateTime=%s", dateTime);
  endTime = dateTime;

  try
  {
    ASSERT(DM != NULL);
    if (!DM->trQuery->InTransaction)
      DM->trQuery->StartTransaction();
    DM->qrQuery->SQL->Clear();
    DM->qrQuery->SQL->Add(
      "insert into CALL(BEGIN_TIME, END_TIME, CALLER, CALLEE, FILE_NAME)"\
      "values (:BEGIN_TIME, :END_TIME, :CALLER, :CALLEE, :FILE_NAME)");

    DM->qrQuery->ParamByName("BEGIN_TIME")->AsString = beginTime.c_str();
    DM->qrQuery->ParamByName("END_TIME")->AsString = endTime.c_str();
    DM->qrQuery->ParamByName("CALLER")->AsString = from.c_str();
    DM->qrQuery->ParamByName("CALLEE")->AsString = to.c_str();
    DM->qrQuery->ParamByName("FILE_NAME")->AsString = extractFileName(waveFileName).c_str();
    // LOG_DEBUG("%s", DM->qrQuery->SQL->Text.c_str()); // gilgil temp 2009.10.10
    DM->qrQuery->ExecSQL();

    /*
    if (!DM->tbCall->Active) DM->tbCall->Open();
    DM->tbCall->Insert();
    DM->tbCall->FieldByName("BEGIN_TIME")->AsString = beginTime.c_str();
    DM->tbCall->FieldByName("END_TIME")->AsString = endTime.c_str();
    DM->tbCall->FieldByName("CALLER")->AsString = from.c_str();
    DM->tbCall->FieldByName("CALLEE")->AsString = to.c_str();
    DM->tbCall->FieldByName("FILE_NAME")->AsString = extractFileName(waveFileName).c_str();
    DM->tbCall->Post();
    */
    DM->trQuery->Commit();
  } catch (Exception& e)
  {
    LOG_FATAL("[VDSniffDB.cpp] VSCallDB::close can not insert data(%s)", e.Message.c_str());
    return true;
  }

  LOG_TRACE("[VDSniffDB.cpp] VSCallDB::close %s inserted to db", waveFileName.c_str());

  return true;
}

// ----------------------------------------------------------------------------
// VSSniffDB
// ----------------------------------------------------------------------------
VSSniffDB::VSSniffDB(void* owner) : VSSniffFile(owner)
{
  // LOG_TRACE("[VSSniffDB.cpp] VSSniffDB::VSSniffDB"); // gilgil temp 2009.10.05
}

VSSniffDB::~VSSniffDB()
{
  // LOG_TRACE("[VSSniffDB.cpp] VSSniffDB::~VSSniffDB"); // gilgil temp 2009.10.05
}

bool VSSniffDB::open()
{
  // LOG_TRACE("[VSSniffDB.cpp] VSSniffDB::open"); // gilgil temp 2009.10.05
  return VSSniffFile::open();
}

bool VSSniffDB::close()
{
  // LOG_TRACE("[VSSniffDB.cpp] VSSniffDB::close"); // gilgil temp 2009.10.05
  return VSSniffFile::close();
}

VSCall* VSSniffDB::createNewCall()
{
  // LOG_TRACE("[VSSniffDB.cpp] VSSniffDB::createNewCall"); // gilgil temp 2009.10.05
  VSCallDB* res = new VSCallDB(this);
  return res;
}

VSSession* VSSniffDB::createNewSession(VSCall* call)
{
  // LOG_TRACE("[VSSniffDB.cpp] VSSniffDB::createNewSession");// gilgil temp 2009.10.05
  return VSSniffFile::createNewSession(call);
}


